#####
#see license.txt
#####
""" Blog Entry Object 2002/11/24 Matt Yarbrough
"""

import os, urllib
from DateTime import DateTime
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.PortalContent import PortalContent, NoWL, ResourceLockedError
from Products.CMFCore.WorkflowCore import WorkflowAction
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFDefault.utils import formatRFC822Headers, html_headcheck
from Products.CMFDefault.utils import SimpleHTMLParser, bodyfinder, parseHeadersBody
from Products.CMFCore.utils import keywordsplitter


import BlogEntryPermissions

#add type to CMF.  You need this.
factory_type_information = (
    {'id': 'PABlog',
     'content_icon': 'blogicon.gif',
     'meta_type': 'PABlog',
     'description': ('Blog entries are diary-ish objects.  Allow Discussion is recommended but not required.'),
     'product': 'PABlog',
     'factory': 'addBlogEntry',
     'immediate_view': 'blogentry_edit_form',
     'actions': ({'id': 'view',
                  'name': 'View',
                  'action': 'blogentry_view',
                  'permissions': (View,)},

		{'id': 'metadata', 
		 'name': 'Metadata', 
		 'action': 'metadata_edit_form', 
		 'permissions': (BlogEntryPermissions.ChangeBlogEntries, )},
                 {'id': 'edit',
                  'name': 'Edit',
                  'action': 'blogentry_edit_form',
                  'permissions': (BlogEntryPermissions.ChangeBlogEntries,)},
                 ),                     
     },
    )
#def, pretty self explanatory.
def addBlogEntry(self
             , id
             , title=''
             , entry_content=''
             , effective_date = None 
             , expiration_date = None 
             , start_date = None 
             , end_date = None
             , mood=''
             , listening=''
             , reading=''
             , watching=''
             , blogentry_url=''
             , REQUEST=None):
  
    
    """
    Create an empty blogentry.
    """
    blogentry = PABlog(id
                  , title
                  , entry_content
                  , effective_date
                  , expiration_date
                  , start_date
                  , end_date
                  , mood
                  , listening
                  , reading
                  , watching
                  , blogentry_url
                 )
    self._setObject(id, blogentry)


# start and end date are here because it makes tying to CMFCalendar really really easy.
# I didn't realize that at first, now I do.  Free archive functionality, sweet.

class PABlog(PortalContent, DefaultDublinCoreImpl):
    """
    Blog entries are diary-ish objects.
    """
    meta_type='PABlog'
    allow_discussion    = 1
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)
    
    __implements__ = ( PortalContent.__implements__
                     , DefaultDublinCoreImpl.__implements__
                     )

    def __init__(self
                 , id
                 , title=''
                 , entry_content=''
                 , effective_date = None 
                 , expiration_date = None
                 , start_date = None
                 , end_date = None 
                 , mood=''
                 , listening=''
                 , reading=''
                 , watching=''
                 , blogentry_url=''
                ):
        DefaultDublinCoreImpl.__init__(self)
        self.id=id
        self.setTitle(title)
        self.setDescription(entry_content)
        self.effective_date = effective_date
        self.expiration_date = expiration_date
        self.setStartDate(start_date)
        if start_date is None:
            start_date = DateTime()
        if end_date is None:
            end_date = start_date
        
        if end_date < start_date:
            end_date = start_date

        self.setEndDate(end_date)
        self.mood=mood
        self.listening=listening
        self.reading=reading
        self.watching=watching
        self.blogentry_url=blogentry_url


    security.declareProtected(BlogEntryPermissions.ChangeBlogEntries, 'edit')
    def edit(self
             , title=None
             , entry_content=None
             , blogentryType=None
             , effectiveDay=None
             , effectiveMo=None
             , effectiveYear=None
             , expirationDay=None
             , expirationMo=None
             , expirationYear=None
             , mood=None
             , listening=None
             , reading=None
             , watching=None
             , blogentry_url=None
            ):

        
        if title is not None: 
            self.setTitle(title)
        if entry_content is not None:
            self.setDescription(entry_content)
        if blogentryType is not None:
            self.setSubject(blogentryType)
        if mood is not None:
            self.mood = mood
        if listening is not None:
            self.listening = listening
        if reading is not None:
            self.reading = reading
        if watching is not None:
            self.watching = watching
        if blogentry_url is not None:
            self.blogentry_url = blogentry_url
        self.reindexObject()
    edit = WorkflowAction(edit) 
    
    security.declarePrivate('handleText')
    def handleText(self, text, format=None):
        """ Handles the raw text, returning headers, body, cooked, format """
        headers = {}
        if format == 'html':
            parser = SimpleHTMLParser()
            parser.feed(text)
            headers.update(parser.metatags)
            if parser.title:
                headers['Title'] = parser.title
            bodyfound = bodyfinder(text)
            if bodyfound:
                body = bodyfound
        else:
            headers, body = parseHeadersBody(text, headers)

        return headers, body, format

    security.declareProtected(ModifyPortalContent, 'setMetadata')
    def setMetadata(self, headers):
        headers['Format'] = self.Format()
        new_subject = keywordsplitter(headers)
        headers['Subject'] = new_subject or self.Subject()
        haveheader = headers.has_key
        for key, value in self.getMetadataHeaders():
            if key != 'Format' and not haveheader(key):
                headers[key] = value
        self._editMetadata(title=headers['Title'],
                          subject=headers['Subject'],
                          entry_content=headers['Description'],
                          contributors=headers['Contributors'],
                          effective_date=headers['Effective_date'],
                          expiration_date=headers['Expiration_date'],
                          format=headers['Format'],
                          language=headers['Language'],
                          rights=headers['Rights'],
                          )
 
    security.declarePublic( 'getMetadataHeaders' )
    def getMetadataHeaders(self):
        """Return RFC-822-style header spec."""
        hdrlist = DefaultDublinCoreImpl.getMetadataHeaders(self)
        hdrlist.append( ('Mood', self.mood) )
        hdrlist.append( ('ListeningTo', self.listening) )
        hdrlist.append( ('Reading', self.reading) )
        hdrlist.append( ('Watching', self.watching) )
        hdrlist.append( ('BlogEntryURL', self.blogentry_url) )

        return hdrlist
    def setStartDate(self, start):
        """
        Setting the event start date, when the event is scheduled to begin.
        """
        self.start_date = self._datify(start)
    
    security.declareProtected(BlogEntryPermissions.ChangeBlogEntries, 'setEndDate')
    def setEndDate(self, end):
        """
        Setting the event end date, when the event ends.
        """
        self.end_date = self._datify(end)

    security.declarePublic('start')
    def start(self):
        """
            Return our start time as a string.
        """
        date = getattr( self, 'start_date', None )
        return date is None and self.created() or date

    security.declarePublic('end')
    def end(self):
        """
            Return our stop time as a string.
        """
        date = getattr( self, 'end_date', None )
        return date is None and self.start() or date    

    security.declarePublic('getStartTimeString')
    def getStartTimeString( self ):
        """
            Return our start time as a string.
        """
        return self.start().AMPMMinutes() 

    security.declarePublic('getStopTimeString')
    def getStopTimeString( self ):
        """
            Return our stop time as a string.
        """
        return self.end().AMPMMinutes() 

# Intialize the BlogEntry class, setting up security.
InitializeClass(PABlog)
